package com.fr.plugin.third.party.jsdidig.data;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.TableData;
import com.fr.data.AbstractDataModel;
import com.fr.general.data.TableDataException;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.third.party.jsdidig.Utils;
import com.fr.plugin.third.party.jsdidig.config.CustomDataConfig;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.utils.Base64;

import java.util.ArrayList;
import java.util.List;

public class CustomHttpDataModel extends AbstractDataModel {
    private static String[] COLUMN_NAMES = {"user_id", "username", "password", "mail", "mobile"};
    private int rowCount = TableData.RESULT_ALL;
    private DatasetData datas = new DatasetData();

    public CustomHttpDataModel(int count) {
        this.rowCount = count;
        if (this.rowCount == 0) {
            return;
        }
        queryData();
    }

    @Override
    public int getColumnCount() throws TableDataException {
        return COLUMN_NAMES.length;
    }

    @Override
    public String getColumnName(int i) throws TableDataException {
        return COLUMN_NAMES[i];
    }

    @Override
    public boolean hasRow(int rowIndex) throws TableDataException {
        int count = getRowCount();
        return rowIndex < count;
    }

    @Override
    public int getRowCount() throws TableDataException {
        if (this.datas == null) {
            return 0;
        }
        List<List<Object>> values = this.datas.getValues();
        if (values == null) {
            return 0;
        }
        int count = values.size();
        return count;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) throws TableDataException {
        if (this.datas == null) {
            return "";
        }
        List<List<Object>> values = this.datas.getValues();
        if ((values == null) || (values.size() <= rowIndex)) {
            return "";
        }
        List<Object> rowValues = values.get(rowIndex);
        if ((rowValues == null) || (rowValues.size() <= columnIndex)) {
            return "";
        }
        return rowValues.get(columnIndex);
    }

    @Override
    public void release() throws Exception {
        this.datas = null;
    }

    /**
     * 查询数据
     */
    private void queryData() {
        try {
            createDatas();
            //if (getRowCount() <= 0) {
            //  throw new NullPointerException("ESB用户数据集获取用户信息为空");
            //}
        } catch (Exception e) {
            LogKit.error("ESB用户数据集获取用户信息,请求出错," + e.getMessage() + "," + e.getCause(), e);
            throw new NullPointerException("ESB用户数据集获取用户信息,请求出错," + e.getMessage() + "," + e.getCause());
        }
    }

    private void createDatas() throws Exception {
        String userSyncUrl = CustomDataConfig.getInstance().getUserSyncUrl();
        String userSyncUsername = CustomDataConfig.getInstance().getUserSyncUsername();
        String userSyncPassword = CustomDataConfig.getInstance().getUserSyncPassword();
        if (StringKit.isEmpty(userSyncUrl) || StringKit.isEmpty(userSyncUsername) || StringKit.isEmpty(userSyncPassword)) {
            return;
        }

        CloseableHttpClient httpClient = Utils.createHttpClient(userSyncUrl);
        String authValue = userSyncUsername + ":" + userSyncPassword;
        String auth = "Basic " + Base64.getEncoder().encodeToString(authValue.getBytes("utf-8"));
        String content = "", cookie = "", syncUrl = "";
        int syncCount = 0;
        do {
            syncUrl = userSyncUrl;
            if (StringKit.isNotEmpty(cookie)) {
                syncUrl = userSyncUrl + "&cookie=" + cookie;
            }
            content = Utils.createHttpGetContent(httpClient, syncUrl, auth);
            if (StringKit.isEmpty(content)) {
                httpClient.close();
                return;
            }
            syncCount++;
            LogKit.info("ESB用户数据集,json内容" + syncCount + ":\n" + content);
            addUsers(content);
            cookie = getCookieValue(content);
            LogKit.info("ESB用户数据集,cookie内容" + syncCount + ":" + cookie);
        } while (StringKit.isNotEmpty(cookie));
        httpClient.close();
    }

    private void addUsers(String content) throws TableDataException {
        String deptName = "", jobTitle = "", userId = "", userName = "", password = "", mobile = "", mail = "", userDesc = "";
        if (StringKit.isEmpty(content)) {
            return;
        }
        JSONObject resJson = new JSONObject(content);
        JSONObject resultInfoJson = resJson.getJSONObject("resultInfo");
        if (resultInfoJson == null) {
            return;
        }
        String success = resultInfoJson.getString("success", "");
        if (!"true".equalsIgnoreCase(success)) {
            return;
        }

        JSONObject bodyJson = resultInfoJson.getJSONObject("body");
        if (bodyJson == null) {
            return;
        }

        JSONArray wsUsersJson = bodyJson.getJSONArray("wsUsers");
        if ((wsUsersJson == null) || (wsUsersJson.size() <= 0)) {
            return;
        }

        JSONArray attributesJson;
        JSONObject attributeJson;
        String userStatus, name, value;
        for (int i = 0, max = wsUsersJson.size() - 1; i <= max; i++) {
            attributesJson = wsUsersJson.getJSONObject(i).getJSONArray("attributes");
            if (attributesJson == null) {
                continue;
            }
            userId = "";
            userName = "";
            password = "123456";
            mail = "";
            mobile = "";
            userStatus = "1";
            for (int j = 0, jMax = attributesJson.size() - 1; j <= jMax; j++) {
                attributeJson = attributesJson.getJSONObject(j);
                name = attributeJson.getString("name");
                if (StringKit.equalsIgnoreCase("alias", name)) {
                    userId = attributeJson.getString("value");
                    continue;
                }
                if (StringKit.equalsIgnoreCase("usercn", name)) {
                    userName = attributeJson.getString("value");
                    continue;
                }
                if (StringKit.equalsIgnoreCase("mail", name)) {
                    mail = attributeJson.getString("value");
                    continue;
                }
                if (StringKit.equalsIgnoreCase("mobile", name)) {
                    mobile = attributeJson.getString("value");
                    continue;
                }
                if (StringKit.equalsIgnoreCase("userstatus", name)) {
                    userStatus = attributeJson.getString("value");
                    continue;
                }
            }

            if (StringKit.isEmpty(userId)) {
                continue;
            }

            if (!StringKit.equalsIgnoreCase("1", userStatus)) {
                continue;
            }

            if ((this.rowCount >= 1) && (getRowCount() >= this.rowCount)) {
                return;
            }
            userId = userId.toLowerCase();
            addRowDatas(userId, userName, password, mail, mobile);
        }
    }

    private String getCookieValue(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject resJson = new JSONObject(content);
        JSONObject resultInfoJson = resJson.getJSONObject("resultInfo");
        if (resultInfoJson == null) {
            return "";
        }
        String success = resultInfoJson.getString("success", "");
        if (!"true".equalsIgnoreCase(success)) {
            return "";
        }

        JSONObject bodyJson = resultInfoJson.getJSONObject("body");
        if (bodyJson == null) {
            return "";
        }

        String cookie = bodyJson.getString("cookie", "");
        if (StringKit.isEmpty(cookie)) {
            return "";
        }
        if ("null".equalsIgnoreCase(cookie)) {
            return "";
        }
        return cookie;
    }

    // {"dept_name", "job_title", "user_id", "username", "password", "mobile", "mail", "code", "fcode"};
    private void addRowDatas(String userId, String userName, String password, String mail, String mobile) {
        List<Object> rowDatas = new ArrayList<>();
        rowDatas.add(userId);
        rowDatas.add(userName);
        rowDatas.add(password);
        rowDatas.add(mail);
        rowDatas.add(mobile);
        List<List<Object>> values = this.datas.getValues();
        values.add(rowDatas);
    }
}
