package com.fr.plugin.third.party.jsdidig.http;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.data.NetworkHelper;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.mobile.terminal.TerminalHandler;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.ComparatorUtils;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;

import com.fr.plugin.third.party.jsdidig.Utils;
import com.fr.plugin.third.party.jsdidig.config.CustomDataConfig;
import com.fr.third.org.apache.http.HttpEntity;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.NameValuePair;
import com.fr.third.org.apache.http.client.config.RequestConfig;
import com.fr.third.org.apache.http.client.entity.UrlEncodedFormEntity;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpGet;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.message.BasicNameValuePair;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.third.springframework.web.util.UriUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


public class SessionGlobalRequestFilterProvider extends AbstractGlobalRequestFilterProvider {
    private static String DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36";

    @Override
    public String filterName() {
        return "com.fr.plugin.third.party.jsdidig";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/decision", "/decision/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            if (!PluginContexts.currentContext().isAvailable()) {
                LogKit.info("集成登录,许可证过期");
                filterChain.doFilter(req, res);
                return;
            }

            String reqUrl = req.getRequestURL().toString();
            String fullUrl = Utils.getFullRequestUrl(req);
            String method = req.getMethod();
            LogKit.info("集成登录,记录访问地址：" + method + " " + fullUrl);

            if (!"GET".equalsIgnoreCase(method)) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/remote/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (reqUrl.indexOf("/decision/login") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("/weixin/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }


            if (fullUrl.indexOf("/dingtalk/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (isAllowIdmOAuthLogin(req)) {
                String requestUrl = getRequestUrl(req);
                requestUrl = replaceUrl(requestUrl);
                LogKit.info("集成登录,真实访问地址," + requestUrl);
                //CustomDataConfig.addUrl(state, requestUrl);
                String locationUrl = Utils.getOAuthCodeUrl(requestUrl);
                Utils.sendRedirect(res, locationUrl);
                return;
            }

            String loginUsername = getIdmOAuthUsername(req);
            if (StringKit.isEmpty(loginUsername)) {
                filterChain.doFilter(req, res);
                return;
            }

            LogKit.info("集成登录,  OAuth 用户名:" + loginUsername);
            User user = UserService.getInstance().getUserByUserName(loginUsername);
            boolean tipsOption = false;
            String tipsContent = "";
            if (user == null) {
                tipsOption = true;
                LogKit.info("集成登录,用户名:" + loginUsername + "在报表平台不存在");
                tipsContent = "在报表服务器上不存在";
            } else if (!user.isEnable()) {
                tipsOption = true;
                LogKit.info("集成登录,用户名:" + loginUsername + "在报表平台上被禁用");
                tipsContent = "在报表平台上被禁用";
            }

            if (tipsOption) {
                String jumpContent = "<!doctype html>\n" +
                        "<head>\n" +
                        "    <meta charset=\"utf-8\" />\n" +
                        "    <title>提示</title>\n" +
                        "<script type=\"text/javascript\">\n" +
                        "window.onload=function(){\n" +
                        "            alert(\"用户:" + loginUsername + tipsContent + ",请联系管理员!\");\n" +
                        "            window.location = \"" + CustomDataConfig.getInstance().getFrUrl() + "\";\n" +
                        "}\n" +
                        "</script>\t\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "    <div style=\"width: 100%;height:200px;  line-height: 200px;font-size:30px;vertical-align:middle;text-align:center\">\n" +
                        "        <span id=\"show\"></span>\n" +
                        "    </div>\n" +
                        "</body>\n" +
                        "</html>";
                res.setContentType("text/html;charset=UTF-8");
                WebUtils.printAsString(res, jumpContent);
                res.setStatus(200);
                return;
            }

            //loginUsername = user.getUserName();
            //LogKit.info("集成登录,报表平台用户名:" + loginUsername);

            LogKit.info("集成登录,报表平台用户名:" + loginUsername + "生成 login token");
            String loginToken = LoginService.getInstance().login(req, res, loginUsername);
            req.setAttribute("fine_auth_token", loginToken);

            //String realUrl = getRealUrl(req);
            //if (StringKit.isEmpty(realUrl)) {
            //    realUrl = CustomDataConfig.getInstance().getFrUrl();
            //}

            if (fullUrl.indexOf("code=") >= 0) {
                String requestUrl = getRequestUrl(req);
                requestUrl = replaceUrl(requestUrl);
                LogKit.info("集成登录,真实跳转地址:" + requestUrl);
                Utils.sendRedirect(res, requestUrl);
                return;
            }
            filterChain.doFilter(req, res);
        } catch (Exception e) {
            LogKit.error("集成登录出错," + e.getMessage(), e);
        }
    }

    private String getRealUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String state = getUrlId(req);
        if (StringKit.isEmpty(state)) {
            return "";
        }
        String url = CustomDataConfig.getInstance().getUrlAndDestroy(state);
        return url;
    }

    String getUrlId(HttpServletRequest req) {
        String urlId = StringKit.trim(WebUtils.getHTTPRequestParameter(req, "url_id"));
        return urlId;
    }

    public boolean isLogged(HttpServletRequest req) {
        boolean logged = true;

        try {
            String token = TokenResource.COOKIE.getToken(req);
            LoginService.getInstance().loginStatusValid(token, TerminalHandler.getTerminal(req, NetworkHelper.getDevice(req)));
        } catch (Exception var4) {
            logged = false;
        }

        return logged;
    }



    private String getMappingUrl(String id) {
        String tempUrl = StringKit.trim(CustomDataConfig.getInstance().getFrUrl());
        if (!tempUrl.endsWith("/")) {
            tempUrl = tempUrl + "/";
        }

        tempUrl = tempUrl + "jsdigei-url/" + id;
        return tempUrl;
    }


    private String replaceUrl(String url) {
        if (StringKit.isEmpty(url)) {
            return CustomDataConfig.getInstance().getFrUrl();
        }

        if (url.indexOf("/decision/login") >= 0) {
            return CustomDataConfig.getInstance().getFrUrl();
        }

        String tempUrl = CustomDataConfig.getInstance().getFrUrl();
        int index = tempUrl.indexOf("/decision");
        if (index < 0) {
            return url;
        }

        String pUrl = tempUrl.substring(0, index);

        index = url.indexOf("/decision");
        if (index < 0) {
            return url;
        }
        String fullUrl = pUrl + url.substring(index);
        return fullUrl;
    }


    /**
     * 判断app是否允许登录IDM OAuth
     *
     * @param req
     * @return
     */
    private boolean isAllowIdmOAuthLogin(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        String loginTypeNameParameter = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String loginTypeConfigValue = CustomDataConfig.getInstance().getLoginTypeValue();
        if (StringKit.isEmpty(loginTypeNameParameter) || StringKit.isEmpty(loginTypeConfigValue)) {
            return false;
        }
        String loginTypeValue = WebUtils.getHTTPRequestParameter(req, loginTypeNameParameter);
        return ComparatorUtils.equals(loginTypeConfigValue, loginTypeValue);
    }


    /**
     * 判断字符串是否全是数字
     *
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        if (StringKit.isEmpty(str)) {
            return false;
        }
        for (int i = str.length(); --i >= 0; ) {
            if (!Character.isDigit(str.charAt(i))) {

                return false;

            }
        }
        return true;
    }


    /**
     * 获取IDM OAuth 用户名
     *
     * @param req
     * @return
     */
    private String getIdmOAuthUsername(HttpServletRequest req) {
        try {
            if (req == null) {
                return "";
            }
            String oAuthCode = WebUtils.getHTTPRequestParameter(req, "code");
            if (StringKit.isEmpty(oAuthCode)) {
                return "";
            }
            LogKit.info("集成登录,code:" + oAuthCode);
            RequestConfig requestConfig = RequestConfig.custom()
                    .setConnectionRequestTimeout(10000)
                    .setSocketTimeout(10000) // 服务端相应超时
                    .setConnectTimeout(10000) // 建立socket链接超时时间
                    .build();


            String accessTokenUrl = CustomDataConfig.getInstance().getAccessTokenUrl();
            LogKit.info("集成登录,获取access_toke url:" + accessTokenUrl);

            HttpPost httpPost = new HttpPost(accessTokenUrl);
            httpPost.addHeader("User-Agent", DEFAULT_USER_AGENT);
            List<NameValuePair> params = new ArrayList<NameValuePair>();
            NameValuePair redirectUriNameValuePair = new BasicNameValuePair("redirect_uri", CustomDataConfig.getInstance().getFrUrl());
            NameValuePair codeNameValuePair = new BasicNameValuePair("code", oAuthCode);
            NameValuePair grantTypeNameValuePair = new BasicNameValuePair("grant_type", "authorization_code");
            NameValuePair clientIdValuePair = new BasicNameValuePair("client_id", CustomDataConfig.getInstance().getIdmClientId());
            NameValuePair clientSecretValuePair = new BasicNameValuePair("client_secret", CustomDataConfig.getInstance().getIdmClientSecret());
            params.add(redirectUriNameValuePair);
            params.add(codeNameValuePair);
            params.add(grantTypeNameValuePair);
            params.add(clientIdValuePair);
            params.add(clientSecretValuePair);
            httpPost.setEntity(new UrlEncodedFormEntity(params, "UTF-8"));
            httpPost.setHeader("Content-type", "application/x-www-form-urlencoded");

            httpPost.setConfig(requestConfig);
            CloseableHttpClient httpClient = Utils.createHttpClient(accessTokenUrl);
            CloseableHttpResponse response = httpClient.execute(httpPost);
            int statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http status:" + statusCode);
                return "";
            }

            HttpEntity httpEntity = response.getEntity();
            if (httpEntity == null) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http响应内容为空");
                return "";
            }
            String responseContent = EntityUtils.toString(httpEntity, "UTF-8");
            response.close();
            if (StringKit.isEmpty(responseContent)) {
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,http响应内容为空1");
                return "";
            }
            LogKit.info("集成登录,获取Access Token请求,http响应内容\n" + responseContent);

            String accessToken = getAccessToken(responseContent);
            if (StringKit.isEmpty(accessToken)) {
                httpClient.close();
                LogKit.info("集成登录,获取Access Token请求出错,access_token为空");
                return "";
            }
            LogKit.info("集成登录,Access Token:" + accessToken);


            
            String userUrl = CustomDataConfig.getInstance().getUserUrl();
            LogKit.info("集成登录,获取用户信息Url:" + userUrl);
            httpPost = new HttpPost(userUrl);
            httpPost.setConfig(requestConfig);
            httpPost.addHeader("User-Agent", DEFAULT_USER_AGENT);

            params.clear();
            NameValuePair accessTokenValuePair = new BasicNameValuePair("access_token", accessToken);
            params.add(accessTokenValuePair);
            httpPost.setEntity(new UrlEncodedFormEntity(params, "UTF-8"));
            httpPost.setHeader("Content-type", "application/x-www-form-urlencoded");
            response = httpClient.execute(httpPost);
            statusCode = response.getStatusLine().getStatusCode();
            if (statusCode != HttpStatus.SC_OK) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取用户信息请求出错,http status:" + statusCode);
                return "";
            }

            httpEntity = response.getEntity();
            if (httpEntity == null) {
                response.close();
                httpClient.close();
                LogKit.info("集成登录,获取用户信息请求出错,http响应内容为空");
                return "";
            }
            responseContent = EntityUtils.toString(httpEntity, "UTF-8");
            response.close();
            httpClient.close();
            if (StringKit.isEmpty(responseContent)) {
                LogKit.info("集成登录,获取用户信息请求出错,http响应内容为空1");
                return "";
            }
            LogKit.info("集成登录,获取用户信息请求,http响应内容\n" + responseContent);

            String uid = getUsername(responseContent);
            if (StringKit.isEmpty(uid)) {
                LogKit.info("集成登录,获取用户信息请求出错,用户名为空");
                return "";
            }
            LogKit.info("集成登录,用户名:" + uid);
            uid = uid.toLowerCase();
            return uid;
        } catch (Exception e) {
            LogKit.error("集成登录获取用户名出错," + e.getMessage(), e);
        }
        return "";
    }

    private String getAccessToken(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject jsonObject = new JSONObject(content);
        String msg = jsonObject.getString("message");
        if (!StringKit.equalsIgnoreCase("SUCCESS", msg)) {
            return "";
        }
        String token = jsonObject.getString("access_token");
        if (StringKit.isNotEmpty(token) && token.startsWith("access_token=")) {
            token = token.substring("access_token=".length());
        }
        return token;
    }

    private String getUsername(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }
        JSONObject jsonObject = new JSONObject(content);
        String msg = jsonObject.getString("message");
        if (!StringKit.equalsIgnoreCase("SUCCESS", msg)) {
            return "";
        }
        JSONObject attributesJson = jsonObject.getJSONObject("attributes");
        if (attributesJson == null) {
            return "";
        }
        String username = attributesJson.getString("smart-alias");
        if (StringKit.isEmpty(username)) {
            return "";
        }
        username = username.toLowerCase();
        return username;
    }


    private String getRequestUrl(HttpServletRequest req) throws UnsupportedEncodingException {
        String fullUrl = req.getRequestURL().toString();
        Map<String, String[]> paraMap = req.getParameterMap();
        String paraName;
        String[] paraValues;
        String loginTypeParaName = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String queryStr = "";
        for (Map.Entry<String, String[]> entry : paraMap.entrySet()) {
            paraName = entry.getKey();
            if (ComparatorUtils.equals(paraName, loginTypeParaName)) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "code")) {
                continue;
            }
            //if (ComparatorUtils.equals(paraName, "url_id")) {
            //    continue;
            //}
            paraValues = entry.getValue();
            LogKit.info("集成登录,获取用户信息请求出错,login_name为空");
            queryStr = addParaToQuery(queryStr, paraName, paraValues);
        }
        if (StringKit.isEmpty(queryStr)) {
            return fullUrl;
        }
        fullUrl = fullUrl + "?" + queryStr;
        return fullUrl;
    }

    private String addParaToQuery(String query, String paraName, String[] paraValues) throws UnsupportedEncodingException {
        if (StringKit.isEmpty(paraName)) {
            return query;
        }
        String fullQuery = query;
        if ((paraValues == null) || (paraValues.length <= 0)) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = paraName + "=";
            return fullQuery;
        }
        String value;
        for (int i = 0, max = paraValues.length - 1; i <= max; i++) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            value = paraValues[i];
            if (StringKit.equals("viewlet", paraName) && (value.indexOf("%") < 0)) {
                value = UriUtils.encodeQueryParam(value, "UTF-8");
            }
            fullQuery = fullQuery + paraName + "=" + value;
        }
        return fullQuery;
    }


}
